--- alloc.c	Sun Jun 08 02:10:41 1997
+++ new/alloc.c	Wed Aug 06 23:18:44 1997
@@ -1,13 +1,25 @@
 /*
  * alloc.c --- allocate new inodes, blocks for ext2fs
  *
- * Copyright (C) 1993 Theodore Ts'o.  This file may be redistributed
- * under the terms of the GNU Public License.
+ * Copyright (C) 1993, 1994, 1995, 1996 Theodore Ts'o.
+ *
+ * %Begin-Header%
+ * This file may be redistributed under the terms of the GNU Public
+ * License.
+ * %End-Header%
+ * 
  */
 
 #include <stdio.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <time.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#if HAVE_ERRNO_H
 #include <errno.h>
-#include <linux/types.h>
+#endif
+
 #include <linux/ext2_fs.h>
 
 #include "ext2fs.h"
@@ -36,16 +48,16 @@
 		dir_group = (dir - 1) / EXT2_INODES_PER_GROUP(fs->super);
 
 	start_inode = (dir_group * EXT2_INODES_PER_GROUP(fs->super)) + 1;
+	if (start_inode < EXT2_FIRST_INODE(fs->super))
+		start_inode = EXT2_FIRST_INODE(fs->super);
 	i = start_inode;
-	if (i < EXT2_FIRST_INO)
-		i = EXT2_FIRST_INO;
 
 	do {
 		if (!ext2fs_test_inode_bitmap(map, i))
 			break;
 		i++;
 		if (i > fs->super->s_inodes_count)
-			i = EXT2_FIRST_INO;
+			i = EXT2_FIRST_INODE(fs->super);
 	} while (i != start_inode);
 	
 	if (ext2fs_test_inode_bitmap(map, i))
@@ -61,7 +73,7 @@
 errcode_t ext2fs_new_block(ext2_filsys fs, blk_t goal,
 			   ext2fs_block_bitmap map, blk_t *ret)
 {
-	blk_t	i = goal;
+	blk_t	i;
 
 	EXT2_CHECK_MAGIC(fs, EXT2_ET_MAGIC_EXT2FS_FILSYS);
 
@@ -69,34 +81,21 @@
 		map = fs->block_map;
 	if (!map)
 		return EXT2_ET_NO_BLOCK_BITMAP;
-	if (!i)
-		i = fs->super->s_first_data_block;
+	if (!goal || (goal >= fs->super->s_blocks_count))
+		goal = fs->super->s_first_data_block;
+	i = goal;
 	do {
 		if (!ext2fs_test_block_bitmap(map, i)) {
 			*ret = i;
 			return 0;
 		}
 		i++;
-		if (i > fs->super->s_blocks_count)
+		if (i >= fs->super->s_blocks_count)
 			i = fs->super->s_first_data_block;
 	} while (i != goal);
 	return ENOSPC;
 }
 
-static int check_blocks_free(ext2_filsys fs, ext2fs_block_bitmap map,
-			     blk_t blk, int num)
-{
-	int	i;
-
-	for (i=0; i < num; i++) {
-		if ((blk+i) > fs->super->s_blocks_count)
-			return 0;
-		if (ext2fs_test_block_bitmap(map, blk+i))
-			return 0;
-	}
-	return 1;
-}
-
 errcode_t ext2fs_get_free_blocks(ext2_filsys fs, blk_t start, blk_t finish,
 				 int num, ext2fs_block_bitmap map, blk_t *ret)
 {
@@ -115,13 +114,13 @@
 	if (!num)
 		num = 1;
 	do {
-		if (check_blocks_free(fs, map, b, num)) {
+		if (b+num-1 > fs->super->s_blocks_count)
+			b = fs->super->s_first_data_block;
+		if (ext2fs_fast_test_block_bitmap_range(map, b, num)) {
 			*ret = b;
 			return 0;
 		}
 		b++;
-		if (b > fs->super->s_blocks_count)
-			b = fs->super->s_first_data_block;
 	} while (b != finish);
 	return ENOSPC;
 }
