/*******************************************************************************
 *
 * dev.c
 *
 * part of ext2 NT tools
 *
 * Copyright (C) Peeter Joot, 1997
 *
 * This file may be redistributed under the terms of the GNU Public License.
 ******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "winperror.h"
#include "dev.h"
#include "ext2fs/ext2_err.h"
#include "linux/ext2_fs.h"

/*******************************************************************************
 *
 * block/superblock related functions.
 *
 ******************************************************************************/

unsigned nt_raw_read(HANDLE dev, LARGE_INTEGER offset, unsigned bytes, char * buf)
{
	OVERLAPPED overl;
	DWORD b_read;

	overl.Offset = offset.LowPart;
	overl.OffsetHigh = offset.HighPart;
	overl.hEvent = NULL;
			
	if ( ReadFile(dev, buf, bytes, &b_read, &overl) == 0 )
	{
		DWORD err = winperror("ReadFile error");
		return err;
	}
	if ( b_read != bytes ) {
		fprintf(stderr,
			"I/O read error %lu of %lu bytes read).\n", b_read
			, b_read, bytes );
		return 2;
	}

	return 0;
}

int ext2_partitiondata(unsigned drivenum, unsigned partition, HANDLE * dev)
{
	char dnamebuf[64];
	char dname[64];
	char dnamel[64];
	int rc;
	
	sprintf(dname,"dev_hd%c%i", 'a' + (char)drivenum, partition); 
	sprintf(dnamel,"\\Device\\Harddisk%i\\Partition%i", drivenum, partition);

	if ( QueryDosDevice(dname, dnamebuf, sizeof(dnamebuf)) == 0) {
		rc = GetLastError();
//		printf("query errcode = %i\n", rc);
		if ( rc == ERROR_FILE_NOT_FOUND ) {
			if ( DefineDosDevice(DDD_RAW_TARGET_PATH, 
				dname, dnamel) == 0 ) 
			return winperror("couldn't define device: %s", dnamel);
		}
		else 
			return winperror("query dos device");
	}

	sprintf(dnamebuf,"\\\\.\\%s", dname);

	*dev = CreateFile(
		dnamebuf
		, GENERIC_READ
		/* | GENERIC_WRITE */
		, FILE_SHARE_READ
		/* | FILE_SHARE_WRITE */
		, NULL
		, OPEN_EXISTING
		, FILE_ATTRIBUTE_READONLY
		/* | FILE_FLAG_RANDOM_ACCESS */
		, NULL
		);

	if ( *dev == INVALID_HANDLE_VALUE )
		return winperror("CreateFile failure");

	return 0;
}

int check_super(HANDLE dev, LARGE_INTEGER sb_offset, struct ext2_super_block *sb)
{
	/* read the super block */

	int rc;	
	if ( (rc = nt_raw_read(dev, sb_offset, 1024, (char*)sb) ))
		fprintf(stderr,"check_super:Couldn't read block at offset %u:%u.\n"
			, sb_offset.HighPart, sb_offset.LowPart );
	else if ( sb->s_magic != EXT2_SUPER_MAGIC ) {
		fprintf(stderr,"bad magic number %hu for super"
			"block.\n", sb->s_magic);
		rc = -1;
		}
	return rc;
}

/****************************************************************************
 * cwdenv_lookup() scans a disk/partition/inode identification in the E2CWD 
 * environment variable.
 *
 * Returns an error code
 ****************************************************************************/

int cwdenv_lookup( e2cwd_t * cwd )
{
	char env[80], *envp, *pos1, *pos2, *cp;

	/* Decode E2CWD environment */
	envp=getenv("E2CWD");
	
	if (!envp)
		return EXT2_ET_NOCWD;

	strcpy(env,envp);			/* We are going to modify it, so we make a copy */
	cwd->name = strdup(envp);

	pos1=strchr(env,':');

	if (!pos1)
		return EXT2_ET_BADCWD;

	pos2=strchr(pos1+1,':');

	*pos1 = 0;
	pos1++;

	for (cp=env; *cp; cp++)
		if (!isdigit(*cp))
			return EXT2_ET_BADCWD;

	cwd->disk = atoi(env);

	if (pos2) {
		*pos2 = 0;
		pos2++;
	}

	for (cp=pos1; *cp; cp++)
		if (!isdigit(*cp))
			return EXT2_ET_BADCWD;

	cwd->part = atoi(pos1);

	if (pos2) {
		for (cp=pos2; *cp; cp++)
			if (!isdigit(*cp))
				return EXT2_ET_BADCWD;

		cwd->cwdino = atoi(pos2);
	}
	else
		cwd->cwdino = EXT2_ROOT_INO;

	return 0;
}
